# -*- coding: utf-8 -*-
# @Time : 2023/07/04
# @Author : chengwenping2
# @File : __init__.py.py
"""
文件说明：
"""
__title__ = 'pig_sdk'
__description__ = '💯一款简洁、好用的接口自动化框架.'
__url__ = 'https://git.oak.net.cn/test_group/pigs_sdk/-/tree/master/pig_frame'
__version__ = "1.3.0",
__author__ = 'chengwenping2'
__author_email__ = 'chengwenping2@newhope.cn'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023 股份总部-数字科技大部-质量技术中心'

